package common;
import java.math.BigInteger;

import mapred.Count;

/**
 * Benchmarking the encryption scheme with addition, multiplication and exponentiation.
 * @author vohuudtr
 *
 */
public class Benchmark {
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		int i;
		long t;
		double te, td, ta, tm, texp;
		BigInteger x = new BigInteger("1");
		BigInteger c = null;
		BigInteger y = null;
		
		Cipher.initialize(Count.class.getResourceAsStream("params.txt"));

		// ENCRYPTION
		t = System.currentTimeMillis();
		for (i = 0; i < 100000; i++) {
			c = Cipher.encrypt(x);
		}
		te = (double)(System.currentTimeMillis()-t)/100000;
		
		// DECRYPTION
		t = System.currentTimeMillis();
		for (i = 0; i < 10000; i++) {
			y = Cipher.decrypt(c);
		}
		td = (double)(System.currentTimeMillis()-t)/10000; 
		
		// ADDITION ON INTEGERS
		y = c;
		t = System.currentTimeMillis();
		for (i = 0; i < 1000000; i++) {
			y = y.add(c);
		}
		ta = (double)(System.currentTimeMillis()-t)/1000000;
		
		// MULTIPLICATION ON INTEGERS
		y = c;
		t = System.currentTimeMillis();
		for (i = 0; i < 10000; i++) {
			y = c.multiply(c);
		}
		tm = (double)(System.currentTimeMillis()-t)/10000; 
		
		// EXPONENTIATION ON INTEGERS
		y = c;
		t = System.currentTimeMillis();
		for (i = 0; i < 1000; i++) {
			y = c.pow(15);
		}
		texp = (double)(System.currentTimeMillis()-t)/1000;
		
		if (args.length == 1 && args[0].equals("short")) {
			System.out.println(Parameters.getP().bitLength() + "," +
					te + "," + 
					td + "," +
					ta + "," +
					tm + "," +
					texp);
		}
		else {
			System.out.println("Size of p: " + Parameters.getP().bitLength());
			System.out.println("Encryption: " + te + "ms");
			System.out.println("Decryption: " + td + "ms");
			System.out.println("Addition on integers: " + ta + "ms");
			System.out.println("Multiplication on integers: " + tm + "ms");
			System.out.println("Exponentiation (^15) on integers: " + texp + "ms");
		}
//		BigInteger c = Cipher.encrypt(new BigInteger("1"));
//		BigInteger count = Cipher.encrypt(new BigInteger("1000"));
//		BigInteger s = new BigInteger("0");
//		t = System.currentTimeMillis();
//		for (i = 0; i < 100000; i++)
//			s = count.multiply(c);
//		t = System.currentTimeMillis() - t;
//		System.out.println((double)t/100000 + "ms " + Cipher.decrypt(s, 2).toString());
//		
//		t = System.currentTimeMillis();
//		for (i = 0; i < 1000; i++) {
//			s = new BigInteger("0");
//			for (int n = 0; n < 1000; n++)
//				s = s.add(c);
//		}
//		t = System.currentTimeMillis() - t;
//		System.out.println((double)t/1000 + "ms " + Cipher.decrypt(s, 1).toString());
	}

}
