package common;

import java.math.BigInteger;
import java.util.Random;

/**
 * Generates prime <code>p</code> and prime <code>q</code> for the 
 * {@link Cipher encryption scheme}.
 * The primes are generated based on the provided security parameters in {@link Parameters}.
 * @author vohuudtr
 *
 */
public class GenPrime {

	/**
	 * Calculates the required size in bits of prime <code>q</code>.
	 * @return number of bits of prime <code>q</code>.
	 */
	public static int calculateSizeQ() {
		double size_q;

		size_q = Math.log(Parameters.getN().longValue()) / Math.log(2);

		if (Math.floor(size_q) == size_q)
			return (int) size_q + 1;
		else
			return (int) Math.ceil(size_q);
	}

	/**
	 * Calculates the required size in bits of prime <code>p</code>.
	 * This calculation requires the calculation of <code>q</code> to be done first.
	 * @param size_q size of <code>q</code>.
	 * @return number of bits of prime <code>p</code>.
	 */
	public static int calculateSizeP(int size_q) {
		double size_p;

		size_p = Parameters.getS1();
		size_p += Math.log(Parameters.getN().longValue()) / Math.log(2);
		size_p += size_q;
		for (int field = 0; field < Parameters.getNumFields(); field++) {
			size_p += (Parameters.getSizeD(field) - 1)
					* (Parameters.getS2() + size_q);
		}

		if (Math.floor(size_p) == size_p)
			return (int) size_p + 1;
		else
			return (int) Math.ceil(size_p);
	}

	/**
	 * Generates prime <code>p</code> and <code>q</code>.
	 */
	public static void generatePrime() {
		int size_p, size_q;

		size_q = calculateSizeQ();
		size_p = calculateSizeP(size_q);

		// generating p and b using the parameters
		Random random = new Random(System.currentTimeMillis());

		Parameters.tic();

		System.out.println("Generating prime p...");
		Parameters.setP(BigInteger.probablePrime(size_p, random));

		System.out.println("Generating random b...");
		Parameters.setB((new BigInteger("2")).modPow(new BigInteger(Parameters
				.getP().bitLength(), random), Parameters.getP()));

		System.out.println("Generating prime q...");
		Parameters.setQ(BigInteger.probablePrime(size_q, random));

		Parameters.toc();
	}

}
