package common;

import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.math.BigInteger;
import java.util.StringTokenizer;

public class LocalCount {
	private Statistics stat = new Statistics();
	private String datafile, origstatfile, resultfile;
	private Polynomial[][] L;

	public LocalCount(String paramfile, String datafile, String origstatfile,
			String resultfile) {
		Cipher.initialize(paramfile);
		stat.initialize(Parameters.getTotalSizeD());
		this.datafile = datafile;
		this.origstatfile = origstatfile;
		this.resultfile = resultfile;
	}

	//	/**
	//	 * Read encrypted big integers from file, decrypt them, and count number of each of them.
	//	 * @throws IOException IO exception.
	//	 */
	//	@SuppressWarnings("deprecation")
	//	public void count() throws IOException {
	//		FileInputStream fis = new FileInputStream(datafile);
	//		DataInputStream dis = new DataInputStream(fis);
	//		BigVector vector_x = new BigVector(Parameters.getSizeD());
	//		BigInteger sumx;
	//		int degree;
	//
	//		vector_x.set(0, new BigInteger("1"));
	//
	//		// read c^k from reduced file and put them into vector_c
	//		while (dis.available() > 0) {
	//			StringTokenizer token = new StringTokenizer(dis.readLine());
	//			degree = Integer.parseInt(token.nextToken());
	//			sumx = Cipher.decrypt(new BigInteger(token.nextToken()), degree);
	//			//			System.out.println("element " + element + " sumx " + sumx);
	//			vector_x.set(degree, sumx);
	//		}
	//		dis.close();
	//		fis.close();
	//
	//		PolyMatrix pm = new PolyMatrix(Parameters.getSizeD());
	//
	//		// count number of appearance
	//		for (int v = 0; v < Parameters.getSizeD(); v++) {
	//			BigVector vector_p = pm.getRow(v);
	//			BigInteger quantity = vector_p.multiply(vector_x).mod(
	//					Parameters.getP());
	//			//			stat.add(v, quantity);
	//		}
	//
	//		stat.print(resultfile);
	//	}

	public void count() {
		int totalSizeD = Parameters.getTotalSizeD();
		MultiArray<BigInteger> sum = MultiArray
				.createBigIntegerArray(totalSizeD);
		MultiArray<BigInteger> origsum = MultiArray
				.createBigIntegerArray(totalSizeD);

		// read encrypted values returned from server
		long t = System.currentTimeMillis();
		try {
			readSum(sum);
			readOrigStat(origsum);
		} catch (IOException e) {
			e.printStackTrace();
			return;
		}
		System.out.println("Decrypting time: " + (System.currentTimeMillis() - t)
				+ "ms");

		// create Lagrange polynomials for each field
		L = new Polynomial[Parameters.getNumFields()][];
		for (int field = 0; field < Parameters.getNumFields(); field++) {
			L[field] = new Polynomial[Parameters.getSizeD(field)];
			for (int v = 0; v < Parameters.getSizeD(field); v++)
				L[field][v] = Polynomial.createLagrange(
						domain(Parameters.getSizeD(field)), v);
//				L[field][v] = Polynomial.createBinaryLagrange(
//						domain(Parameters.getSizeD(field)), v);
		}

		// compute counting number for each point
		t = System.currentTimeMillis();
		for (int i = 0; i < totalSizeD; i++) {
			if (origsum.get(i) != null) {
				stat.add(MultiArray.getMultiFieldKey(i), compute(i, sum)
						.intValue());
//				stat.add(MultiArray.getMultiFieldKey(i), computeBinary(i, sum));
			}
		}
		System.out.println("Counting time: "
				+ (System.currentTimeMillis() - t) + "ms");

		stat.print(resultfile);
	}

	public int[] domain(int size) {
		int[] x = new int[size];
		for (int k = 0; k < x.length; k++)
			x[k] = k;
		return x;
	}

	public int computeBinary(int ind, MultiArray<BigInteger> sum) {
		int[][] f = new int[Parameters.getNumFields()][2];
		MultiFieldKey key = MultiArray.getMultiFieldKey(ind);
		for (int field = 0; field < Parameters.getNumFields(); field++) {
			f[field][0] = L[field][key.element[field]].getCoefficient(0)
					.intValue();
			f[field][1] = L[field][key.element[field]].getCoefficient(1)
					.intValue();
		}

		int count = 0;
		BigInteger[] sum_items = sum.getAll();
		MultiFieldKey k = new MultiFieldKey(Parameters.getNumFields());
		for (int i = 0; i < sum_items.length; i++) {
			BigInteger value = sum_items[i];
			if (value != null) {
				int v = value.intValue();
				if (v != 0) {
					int coeff = 1;
					for (int j = 0; j < k.element.length; j++) {
						if (f[j][k.element[j]] == 0) {
							coeff = 0;
							break;
						}
						coeff = coeff * f[j][k.element[j]];
					}
					count += coeff * v;
				}
			}

			// increase key
			int j = Parameters.getNumFields() - 1;
			k.element[j]++;
			while (k.element[j] == 2) {
				k.element[j] = 0;
				j--;
				if (j < 0)
					break;
				k.element[j]++;
			}
		}

		return count;
	}

	public BigInteger compute(int ind, MultiArray<BigInteger> sum) {
		Polynomial[] f = new Polynomial[Parameters.getNumFields()];
		MultiFieldKey key = MultiArray.getMultiFieldKey(ind);
		for (int field = 0; field < Parameters.getNumFields(); field++) {
			f[field] = L[field][key.element[field]];
		}

		MultiArray<BigInteger> coeffs = MultiArray
				.createBigIntegerArray(Parameters.getTotalSizeD());
		computeCoeffs(f, coeffs, new MultiFieldKey(Parameters.getNumFields()),
				new BigInteger("1"), 0);

		BigInteger count = new BigInteger("0");
		BigInteger[] coeff_items = coeffs.getAll();
		BigInteger[] sum_items = sum.getAll();
		for (int i = 0; i < coeff_items.length; i++) {
			BigInteger coeff = coeff_items[i];
			BigInteger value = sum_items[i];
			if (value != null && coeff != null) {
				count = count.add(coeff.multiply(value)).mod(Parameters.getQ());
			}
		}

		return count;
	}

	public void computeCoeffs(Polynomial[] f, MultiArray<BigInteger> coeffs,
			MultiFieldKey runningKey, BigInteger runningCoeff, int runningField) {
		for (int v = 0; v < Parameters.getSizeD(runningField); v++) {
			BigInteger c = f[runningField].getCoefficient(v);
			if (c.signum() != 0) {
				runningKey.element[runningField] = v;
				if (runningField == Parameters.getNumFields() - 1) {
					coeffs.put(runningKey,
							runningCoeff.multiply(c).mod(Parameters.getQ()));
				} else {
					computeCoeffs(f, coeffs, runningKey,
							runningCoeff.multiply(c).mod(Parameters.getQ()),
							runningField + 1);
				}
			}
		}
	}

	@SuppressWarnings("deprecation")
	public void readSum(MultiArray<BigInteger> sum) throws IOException {
		FileInputStream fis = new FileInputStream(datafile);
		DataInputStream dis = new DataInputStream(fis);

		while (dis.available() > 0) {
			StringTokenizer token = new StringTokenizer(dis.readLine());
			MultiFieldKey key = new MultiFieldKey(Parameters.getNumFields());
			int degree = 0;
			for (int j = 0; j < Parameters.getNumFields(); j++) {
				key.element[j] = Integer.parseInt(token.nextToken());
				degree += key.element[j];
			}

			BigInteger c = new BigInteger(token.nextToken());

			sum.put(key, Cipher.decrypt(c, degree));
		}

		dis.close();
		fis.close();
	}

	@SuppressWarnings("deprecation")
	public void readOrigStat(MultiArray<BigInteger> origsum) throws IOException {
		FileInputStream fis = new FileInputStream(origstatfile);
		DataInputStream dis = new DataInputStream(fis);

		while (dis.available() > 0) {
			StringTokenizer token = new StringTokenizer(dis.readLine());
			MultiFieldKey key = new MultiFieldKey(Parameters.getNumFields());
			for (int j = 0; j < Parameters.getNumFields(); j++) {
				key.element[j] = Integer.parseInt(token.nextToken());
			}

			BigInteger c = new BigInteger(token.nextToken());

			origsum.put(key, c);
		}

		dis.close();
		fis.close();
	}

	public static void usage() {
		System.out.println("Client counting.");
		System.out.println("Usage: java " + LocalCount.class.getName()
				+ " <paramfile> <datafile> <origstatfile> <resultfile>");
		System.out
				.println("   paramfile    Specifies location of the parameter file.");
		System.out
				.println("   datafile     Specifies location of the file storing the encrypted values.");
		System.out
				.println("   origstatfile Specifies location of the original statistic file.");
		System.out
				.println("   resultfile   Specifies location of the file storing final results.");
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		if (args.length == 4) {
			LocalCount lc = new LocalCount(args[0], args[1], args[2], args[3]);
			lc.count();
			System.exit(0);
		}

		usage();
		System.exit(1);
	}

}
