package common;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.math.BigInteger;

import mapred.BigIntegerWritable;

/**
 * @author vohuudtr
 *
 */
public class MultiArray<T> {
	private T[] items;

	private MultiArray(int size) {
	}

	public static MultiArray<BigIntegerWritable> createBigIntegerWritableArray(
			int size) {
		MultiArray<BigIntegerWritable> ma = new MultiArray<BigIntegerWritable>(
				size);
		ma.items = new BigIntegerWritable[size];
		return ma;
	}

	public static MultiArray<BigInteger> createBigIntegerArray(int size) {
		MultiArray<BigInteger> ma = new MultiArray<BigInteger>(size);
		ma.items = new BigInteger[size];
		return ma;
	}

	public static MultiArray<Integer> createIntegerArray(int size) {
		MultiArray<Integer> ma = new MultiArray<Integer>(size);
		ma.items = new Integer[size];
		return ma;
	}

	public static MultiArray<BigIntegerWritable> readMultiArrayBigIntegerWritable(
			DataInput in) throws IOException {
		MultiArray<BigIntegerWritable> a = MultiArray
				.createBigIntegerWritableArray(in.readInt());
		for (int i = 0; i < a.items.length; i++) {
			byte b = in.readByte();
			if (b != 0) {
				a.items[i] = new BigIntegerWritable(in);
			}
		}
		return a;
	}

	public static void write(MultiArray<BigIntegerWritable> a, DataOutput out)
			throws IOException {
		out.writeInt(a.items.length);
		for (int i = 0; i < a.items.length; i++) {
			if (a.items[i] != null) {
				out.writeByte(1);
				a.items[i].write(out);
			} else {
				out.writeByte(0);
			}
		}
	}

	public T get(MultiFieldKey key) {
		return items[getIndex(key)];
	}

	public T get(int ind) {
		return items[ind];
	}

	public void put(MultiFieldKey key, T value) {
		items[getIndex(key)] = value;
	}

	public void put(int ind, T value) {
		items[ind] = value;
	}

	public T[] getAll() {
		return items;
	}

	public int getSize() {
		return items.length;
	}

	public static int getIndex(MultiFieldKey key) {
		int ind = 0;
		for (int i = 0; i < key.element.length; i++) {
			ind = ind * Parameters.getSizeD(i) + key.element[i];
		}
		return ind;
	}

	public static MultiFieldKey getMultiFieldKey(int ind) {
		MultiFieldKey key = new MultiFieldKey(Parameters.getNumFields());
		for (int i = Parameters.getNumFields() - 1; i >= 0; i--) {
			key.element[i] = ind % Parameters.getSizeD(i);
			ind /= Parameters.getSizeD(i);
		}

		return key;
	}
}
