package common;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

import org.apache.hadoop.io.Writable;
import org.apache.hadoop.io.WritableComparable;

/**
 * @author vohuudtr
 *
 */
public class MultiFieldKey implements Cloneable, Writable,
		WritableComparable<MultiFieldKey> {
	public int[] element; /* size of this array is the number of variables.
							element[i] denotes the exponent of the i-th variable. */

	public MultiFieldKey() {

	}

	public MultiFieldKey(int numFields) {
		element = new int[numFields];
		for (int i = 0; i < numFields; i++)
			element[i] = 0;
	}
	
	public MultiFieldKey(DataInput in) throws IOException {
		readFields(in);
	}

	public int weight() {
		int w = 0;
		for (int i = 0; i < element.length; i++)
			w += element[i];
		return w;
	}
	
	@Override
	public boolean equals(Object k) {
		if (k instanceof MultiFieldKey) {
			MultiFieldKey key = (MultiFieldKey) k;
			for (int i = 0; i < element.length; i++)
				if (element[i] != key.element[i])
					return false;
			return true;
		}
		return false;
	}

	@Override
	public int hashCode() {
		int hash = 0;
		for (int i = 0; i < element.length; i++)
			hash += element[i];
		return hash;
	}

	@Override
	public String toString() {
		String s = null;
		for (int i = 0; i < element.length; i++) {
			if (i == 0)
				s = Integer.toString(element[i]);
			else
				s = s + "\t" + element[i];
		}
		return s;
	}

	@Override
	public void readFields(DataInput in) throws IOException {
		element = new int[in.readInt()];
		for (int i = 0; i < element.length; i++)
			element[i] = in.readInt();
	}

	@Override
	public void write(DataOutput out) throws IOException {
		out.writeInt(element.length);
		for (int i = 0; i < element.length; i++)
			out.writeInt(element[i]);
	}

	@Override
	public int compareTo(MultiFieldKey key) {
		for (int i = 0; i < element.length; i++) {
			if (element[i] < key.element[i])
				return -1;
			if (element[i] > key.element[i])
				return 1;
		}
		return 0;
	}

	@Override
	public MultiFieldKey clone() {
		MultiFieldKey newkey = new MultiFieldKey();
		newkey.element = element.clone();
		return newkey;
	}

}
