package common;

import java.io.FileNotFoundException;
import java.io.OutputStream;
import java.io.PrintStream;

/**
 * Provides a tool for collecting statistics about the values of fields in the data set.
 * 
 * @author vohuudtr
 *
 */
public class Statistics {
	private MultiArray<Integer> counter;

	/**
	 * Constructs the object without initialization. 
	 */
	public Statistics() {
	}

	/**
	 * Initializes the object with total number of distinct values in the data set.
	 * @param size number of distinct values in the data set.
	 */
	public void initialize(int size) {
		counter = MultiArray.createIntegerArray(size);
	}
	
	/**
	 * Add and parse the record, and update the statistics.
	 * @param record record to add.
	 */
	public void add(Record record) {
		MultiFieldKey key = new MultiFieldKey(record.getNumFields() / 2);
		for (int j = 0; j < key.element.length; j++) {
			key.element[j] = Integer.parseInt(record.getField(j * 2 + 1).toString());
		}
		add(key, 1);
	}
	
	/**
	 * Add explicitly a value with given quantity.
	 * @param key
	 * @param quantity
	 */
	public void add(MultiFieldKey key, int quantity) {
		Integer count = counter.get(key);
		if (count == null)
			counter.put(key, quantity);
		else
			counter.put(key, count + quantity);
	}

	/**
	 * Prints statistics to given output stream.
	 * @param out output stream.
	 */
	public void print(OutputStream out) {
		PrintStream ps = new PrintStream(out);
		Integer[] items = counter.getAll();
		for (int i = 0; i < items.length; i++) {
			if (items[i] != null && items[i] != 0) {
				ps.println(MultiArray.getMultiFieldKey(i) + "\t" + items[i]);
			}
		}
	}

	/**
	 * Prints statistics to standard output stream.
	 */
	public void print() {
		print(System.out);
	}

	/**
	 * Print statistics to a given file.
	 * @param filename output filename.
	 */
	public void print(String filename) {
		if (filename == null || filename.isEmpty())
			return;

		try {
			print(new PrintStream(filename));
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
	}
}
