/**
 * @mainpage EPiC documentation
 * 
 * @section Abstract
 *   In the face of an untrusted cloud infrastructure, outsourced data
  needs to be protected. Fully homomorphic encryption is one solution
  that also allows performing operations on outsourced data.  However,
  the involved high overhead of today's fully homomorphic encryption
  techniques outweigh cloud cost saving advantages, rendering it
  impractical. We present EPiC, a practical, efficient protocol for
  the privacy-preserving evaluation of a fundamental operation on data
  sets: frequency counting.  In an IND-CPA encrypted outsourced data
  set, a cloud user can specify a pattern, and the cloud will count
  the number of occurrences of this pattern in a completely oblivious
  manner. A pattern is expressed as a boolean formula on the fields 
  of the records and can specify values counting, range counting, and 
  conjunctions/disjunctions of field values. EPiC's main idea is, 
  first, to reduce the problem of counting to a summation of polynomial evaluations. Second, to efficiently evaluate
  the summation of polynomial evaluations in a privacy-preserving manner, we extend previous work
  on the Hidden Modular Group Order assumption and design a new
  <em>somewhat homomorphic</em> encryption scheme. We show how a 
  general pattern, defined by a boolean formula, is arithmetized 
  into a multivariate polynomial over <code>GF(2)</code> and used in EPiC. This scheme is highly
  efficient in our particular counting scenario. Besides a formal analysis where
  we prove EPiC's privacy, we also present implementation and
  evaluation results. We specifically target Google's prominent
  MapReduce paradigm as offered by major cloud providers.  Our
  evaluation performed both locally and in Amazon's public cloud with
  data sets sizes of up to 1 TByte shows only modest overhead compared
  to non-private counting, attesting to EPiC's efficiency.
  
 * @section Details
 * Please read the details <a href="http://eprint.iacr.org/2012/452">here</a>.
 * 
 * @section Implementation
 * The implementation is done in Java. Please use the navigator above to view the packges, classes and methods. 
 */

/**
 * Provides classes needed for MapReduce jobs.
 * <p>
 * The main executable class is {@link mapred.Count Count}, which receives command-line parameters and executes the
 * MapReduce counting job.
 * <p>
 * Different counting approaches are implemented in classes prefixed with <code>MapRed</code>.
 * <p>
 * The {@link mapred.Producer Producer} class is an implementation of a MapReduce job used to generate a large data
 * set in parallel to reduce the generating time.
 */
package mapred;