package obsolete;

import java.math.BigInteger;

import common.Polynomial;

@Deprecated
public class BigVector {
	private BigInteger[] elements;

	/**
	 * Constructing a big vector with given size and initializing all elements
	 * to 0.
	 * 
	 * @param size
	 *            size of the vector.
	 */
	public BigVector(int size) {
		elements = new BigInteger[size];
		for (int i = 0; i < size; i++)
			elements[i] = new BigInteger("0");
	}

	/**
	 * Constructing a big vector from a given big-vector.
	 * 
	 * @param v
	 *            another big-vector.
	 */
	public BigVector(BigVector v) {
		elements = new BigInteger[v.elements.length];
		for (int i = 0; i < elements.length; i++)
			elements[i] = new BigInteger(v.elements[i].toByteArray());
	}

	/**
	 * Constructing a big vector with given big-integer values.
	 * 
	 * @param values
	 *            initial big-integer values.
	 */
	public BigVector(BigInteger[] values) {
		elements = new BigInteger[values.length];
		for (int i = 0; i < values.length; i++)
			elements[i] = new BigInteger(values[i].toByteArray());
	}

	/**
	 * Constructing a big vector with given small-integer values.
	 * 
	 * @param values
	 *            initial small-integer values.
	 */
	public BigVector(int[] values) {
		elements = new BigInteger[values.length];
		for (int i = 0; i < values.length; i++)
			elements[i] = new BigInteger(Integer.toString(values[i]));
	}

	/**
	 * Constructing a big vector with coefficients of given polynomial as
	 * initial values.
	 * 
	 * @param p
	 *            polynomial whose coefficients used as initial values for the
	 *            vector.
	 */
	public BigVector(Polynomial p) {
		elements = new BigInteger[p.degree() + 1];
		for (int i = 0; i < elements.length; i++) {
			elements[i] = new BigInteger(p.getCoefficient(i).toByteArray());
		}
	}

	/**
	 * Return value of the element at "which" position.
	 * 
	 * @param which
	 *            position of the requested element.
	 * @return value of the requested element.
	 */
	public BigInteger get(int which) {
		return elements[which];
	}

	/**
	 * Set value for the element at "which" position.
	 * 
	 * @param which
	 *            position to be set.
	 * @param value
	 *            value to be set.
	 */
	public void set(int which, BigInteger value) {
		elements[which] = new BigInteger(value.toByteArray());
	}

	/**
	 * Perform scalar product between two big vectors and return the result.
	 * 
	 * @param v
	 *            the vector to be multiplied.
	 * @return scalar product of two vectors.
	 */
	public BigInteger multiply(BigVector v) {
		BigInteger r = new BigInteger("0");
		int h = v.elements.length < this.elements.length ? v.elements.length
				: this.elements.length;

		for (int i = 0; i < h; i++) {
			r = r.add(this.elements[i].multiply(v.elements[i]));
		}

		return r;
	}

	@Override
	public String toString() {
		String s = new String("[");

		for (int i = 0; i < elements.length; i++)
			if (i == 0)
				s = s + elements[i].toString();
			else
				s = s + " " + elements[i].toString();
		s = s + "]";

		return s;
	}
}
